import { IRouterSlot, PathFragment } from "./model";
/**
 * Router link.
 * @slot - Default content.
 */
export declare class RouterLink extends HTMLElement {
    private listeners;
    private _context;
    static get observedAttributes(): string[];
    /**
     * The path of the navigation.
     * @attr
     */
    set path(value: string | PathFragment);
    get path(): string | PathFragment;
    /**
     * Whether the element is disabled or not.
     * @attr
     */
    get disabled(): boolean;
    set disabled(value: boolean);
    /**
     * Whether the element is active or not.
     * @attr
     */
    get active(): boolean;
    set active(value: boolean);
    /**
     * Whether the focus should be delegated.
     * @attr
     */
    get delegateFocus(): boolean;
    set delegateFocus(value: boolean);
    /**
     * Whether the query should be preserved or not.
     * @attr
     */
    get preserveQuery(): boolean;
    set preserveQuery(value: boolean);
    /**
     * The current router slot context.
     */
    get context(): IRouterSlot | null;
    set context(value: IRouterSlot | null);
    /**
     * Returns the absolute path.
     */
    get absolutePath(): string;
    constructor();
    /**
     * Hooks up the element.
     */
    connectedCallback(): void;
    /**
     * Tear down listeners.
     */
    disconnectedCallback(): void;
    /**
     * Reacts to attribute changed callback.
     * @param name
     * @param oldValue
     * @param newValue
     */
    attributeChangedCallback(name: string, oldValue: unknown, newValue: unknown): void;
    private updateTabIndex;
    /**
     * Returns the absolute path constructed relative to the context.
     * If no router parent was found the path property is the absolute one.
     */
    constructAbsolutePath(path: string): string;
    /**
     * Updates whether the route is active or not.
     */
    protected updateActive(): void;
    /**
     * Navigates to the specified path.
     */
    navigate(path: string, e?: Event): void;
}
declare global {
    interface HTMLElementTagNameMap {
        "router-link": RouterLink;
    }
}
